@@ -57,14 +57,6 @@
         <source>C&amp;hoose</source>
         <translation type="unfinished">छनौट गर्नुहोस्...</translation>
     </message>
-    <message>
-        <source>Sending addresses</source>
-        <translation type="unfinished">पठाउने ठेगानाहरू</translation>
-    </message>
-    <message>
-        <source>Receiving addresses</source>
-        <translation type="unfinished">प्राप्त गर्ने ठेगानाहरू...</translation>
-    </message>
     <message>
         <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
         <translation type="unfinished">यी भुक्तानी गर्नका लागि तपाइका बिट्कोइन ठेगानाहरू हुन्। सिक्काहरू पठाउनुअघि रकम र प्राप्त गर्ने ठेगाना जाँच गर्नुहोस।</translation>
@@ -95,6 +87,14 @@
         <extracomment>An error message. %1 is a stand-in argument for the name of the file we attempted to save to.</extracomment>
         <translation type="unfinished">ठेगाना सुची %1मा बचत गर्ने प्रयासमा त्रुटि भएको छ। कृपया पुनः प्रयास गर्नुहोस।</translation>
     </message>
+    <message>
+        <source>Sending addresses - %1</source>
+        <translation type="unfinished">ठेगानाहरू पठाउँदै - %1</translation>
+    </message>
+    <message>
+        <source>Receiving addresses - %1</source>
+        <translation type="unfinished">ठेगानाहरू प्राप्त गर्दै - %1</translation>
+    </message>
     <message>
         <source>Exporting Failed</source>
         <translation type="unfinished">निर्यात असफल</translation>
@@ -110,7 +110,11 @@
         <source>Address</source>
         <translation type="unfinished">ठेगाना</translation>
     </message>
-    </context>
+    <message>
+        <source>(no label)</source>
+        <translation type="unfinished">(लेबल छैन)</translation>
+    </message>
+</context>
 <context>
     <name>AskPassphraseDialog</name>
     <message>
@@ -137,6 +141,10 @@
         <source>Encrypt wallet</source>
         <translation type="unfinished">वालेट इन्क्रिप्ट गर्नुहोस् </translation>
     </message>
+    <message>
+        <source>This operation needs your wallet passphrase to unlock the wallet.</source>
+        <translation type="unfinished">यो अपरेसनलाई वालेट अनलक गर्न तपाईंको वालेट पासफ्रेज चाहिन्छ।</translation>
+    </message>
     <message>
         <source>Unlock wallet</source>
         <translation type="unfinished">वालेट अनलक गर्नुहोस्</translation>
@@ -157,6 +165,26 @@
         <source>Wallet encrypted</source>
         <translation type="unfinished">वालेट इन्क्रिप्ट भयो</translation>
     </message>
+    <message>
+        <source>Enter the new passphrase for the wallet.&lt;br/&gt;Please use a passphrase of &lt;b&gt;ten or more random characters&lt;/b&gt;, or &lt;b&gt;eight or more words&lt;/b&gt;.</source>
+        <translation type="unfinished">वालेटको लागि नयाँ पासफ्रेज प्रविष्ट गर्नुहोस्। &lt;br/&gt;कृपया पासफ्रेज प्रयोग गर्नुहोस् &lt;b&gt;दस वा बढी अनियमित वर्णहरू &lt;/b&gt;, वा &lt;b&gt;आठ वा बढी शब्दहरू &lt;/b&gt;.</translation>
+    </message>
+    <message>
+        <source>Enter the old passphrase and new passphrase for the wallet.</source>
+        <translation type="unfinished">वालेटको लागि पुरानो पासफ्रेज र नयाँ पासफ्रेज प्रविष्ट गर्नुहोस्।</translation>
+    </message>
+    <message>
+        <source>Remember that encrypting your wallet cannot fully protect your bitcoins from being stolen by malware infecting your computer.</source>
+        <translation type="unfinished">याद गर्नुहोस् कि तपाईको वालेट इन्क्रिप्ट गर्नाले तपाईको बिटकोइनलाई तपाईको कम्प्युटरमा मालवेयरले चोरी हुनबाट पूर्णतया सुरक्षित गर्न सक्दैन।</translation>
+    </message>
+    <message>
+        <source>Wallet to be encrypted</source>
+        <translation type="unfinished">वालेट इन्क्रिप्ट गर्न</translation>
+    </message>
+    <message>
+        <source>Your wallet is about to be encrypted. </source>
+        <translation type="unfinished">तपाईंको वालेट इन्क्रिप्ट हुन लागेको छ।</translation>
+    </message>
     <message>
         <source>Your wallet is now encrypted. </source>
         <translation type="unfinished">अब वालेट इन्क्रिप्ट भएको छ।</translation>
@@ -165,10 +193,30 @@
         <source>Wallet encryption failed</source>
         <translation type="unfinished">वालेट इन्क्रिप्सन असफल </translation>
     </message>
+    <message>
+        <source>Wallet encryption failed due to an internal error. Your wallet was not encrypted.</source>
+        <translation type="unfinished">आन्तरिक त्रुटिका कारण वालेट इन्क्रिप्सन असफल भयो। तपाईंको वालेट इन्क्रिप्ट गरिएको थिएन।</translation>
+    </message>
+    <message>
+        <source>The supplied passphrases do not match.</source>
+        <translation type="unfinished">प्रदान गरिएका पासफ्रेजहरू मेल खाँदैनन्।</translation>
+    </message>
     <message>
         <source>Wallet unlock failed</source>
         <translation type="unfinished">वालेट अनलक असफल </translation>
     </message>
+    <message>
+        <source>The passphrase entered for the wallet decryption was incorrect.</source>
+        <translation type="unfinished">वालेट डिक्रिप्शनको लागि प्रविष्ट गरिएको पासफ्रेज गलत थियो।</translation>
+    </message>
+    <message>
+        <source>Wallet passphrase was successfully changed.</source>
+        <translation type="unfinished">वालेट पासफ्रेज सफलतापूर्वक परिवर्तन गरियो।</translation>
+    </message>
+    <message>
+        <source>Passphrase change failed</source>
+        <translation type="unfinished">पासफ्रेज परिवर्तन असफल भयो</translation>
+    </message>
     <message>
         <source>Warning: The Caps Lock key is on!</source>
         <translation type="unfinished">चेतावनी: क्याप्स लक कीप्याड अन छ!</translation>
@@ -187,10 +235,18 @@
 </context>
 <context>
     <name>BitcoinApplication</name>
+    <message>
+        <source>Settings file %1 might be corrupt or invalid.</source>
+        <translation type="unfinished">सेटिङ फाइल %1 भ्रष्ट वा अवैध हुन सक्छ।</translation>
+    </message>
     <message>
         <source>Runaway exception</source>
         <translation type="unfinished">रनअवे अपवाद</translation>
     </message>
+    <message>
+        <source>A fatal error occurred. %1 can no longer continue safely and will quit.</source>
+        <translation type="unfinished">एउटा घातक त्रुटि भयो। %1 अब सुरक्षित रूपमा जारी राख्न सक्दैन र छोड्नेछ।</translation>
+    </message>
     <message>
         <source>Internal error</source>
         <translation type="unfinished">आन्तरिक दोष</translation>
@@ -198,6 +254,19 @@
     </context>
 <context>
     <name>QObject</name>
+    <message>
+        <source>Do you want to reset settings to default values, or to abort without making changes?</source>
+        <extracomment>Explanatory text shown on startup when the settings file cannot be read. Prompts user to make a choice between resetting or aborting.</extracomment>
+        <translation type="unfinished">के तपाइँ पूर्वनिर्धारित मानहरूमा सेटिङहरू रिसेट गर्न चाहनुहुन्छ, वा परिवर्तन नगरी रद्द गर्न चाहनुहुन्छ?</translation>
+    </message>
+    <message>
+        <source>Error: %1</source>
+        <translation type="unfinished">त्रुटि: %1</translation>
+    </message>
+    <message>
+        <source>%1 didn't yet exit safely…</source>
+        <translation type="unfinished">%1अझै सुरक्षित बाहिर निस्किएन...</translation>
+    </message>
     <message>
         <source>unknown</source>
         <translation type="unfinished">थाहा नभयेको</translation>
@@ -390,6 +459,10 @@
         <source>&amp;Help</source>
         <translation type="unfinished">&amp;मद्दत</translation>
     </message>
+    <message>
+        <source>Tabs toolbar</source>
+        <translation type="unfinished">ट्याबहरू उपकरणपट्टी</translation>
+    </message>
     <message numerus="yes">
         <source>Processed %n block(s) of transaction history.</source>
         <translation type="unfinished">
@@ -422,6 +495,10 @@
             <numerusform />
         </translation>
     </message>
+    <message>
+        <source>Error: %1</source>
+        <translation type="unfinished">त्रुटि: %1</translation>
+    </message>
     </context>
 <context>
     <name>CoinControlDialog</name>
@@ -442,12 +519,8 @@
         <translation type="unfinished">पुष्टि भयो</translation>
     </message>
     <message>
-        <source>yes</source>
-        <translation type="unfinished">हो</translation>
-    </message>
-    <message>
-        <source>no</source>
-        <translation type="unfinished">होइन</translation>
+        <source>(no label)</source>
+        <translation type="unfinished">(लेबल छैन)</translation>
     </message>
     </context>
 <context>
@@ -469,7 +542,7 @@
     </message>
     <message>
         <source>&amp;Address</source>
-        <translation type="unfinished">&amp;ठेगाना </translation>
+        <translation type="unfinished">&amp;ठेगाना</translation>
     </message>
     <message>
         <source>Could not unlock wallet.</source>
@@ -539,7 +612,7 @@
     </message>
     <message>
         <source>Unknown…</source>
-        <translation type="unfinished">थाहा नभाको</translation>
+        <translation type="unfinished">थाहा नभाको...</translation>
     </message>
     <message>
         <source>calculating…</source>
@@ -656,7 +729,7 @@
     </message>
     <message>
         <source>Close</source>
-        <translation type="unfinished">बन्द गर्नुहोस् </translation>
+        <translation type="unfinished">बन्द गर्नुहोस्</translation>
     </message>
     </context>
 <context>
@@ -731,6 +804,10 @@
         <source>Label</source>
         <translation type="unfinished">लेबल</translation>
     </message>
+    <message>
+        <source>(no label)</source>
+        <translation type="unfinished">(लेबल छैन)</translation>
+    </message>
     </context>
 <context>
     <name>SendCoinsDialog</name>
@@ -745,7 +822,11 @@
             <numerusform />
         </translation>
     </message>
-    </context>
+    <message>
+        <source>(no label)</source>
+        <translation type="unfinished">(लेबल छैन)</translation>
+    </message>
+</context>
 <context>
     <name>SendCoinsEntry</name>
     <message>
@@ -820,6 +901,10 @@
         <source>Label</source>
         <translation type="unfinished">लेबल</translation>
     </message>
+    <message>
+        <source>(no label)</source>
+        <translation type="unfinished">(लेबल छैन)</translation>
+    </message>
     </context>
 <context>
     <name>TransactionView</name>
