@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) 2017-2019 The Bitcoin Core developers
+# Copyright (c) 2017-2021 The Bitcoin Core developers
 # Distributed under the MIT software license, see the accompanying
 # file COPYING or http://www.opensource.org/licenses/mit-license.php.
 """Combine logs from multiple bitcoin nodes as well as the test_framework log.
@@ -188,7 +188,7 @@ def print_logs_plain(log_events, colors):
 def print_logs_html(log_events):
     """Renders the iterator of log events into html."""
     try:
-        import jinja2
+        import jinja2 #type:ignore
     except ImportError:
         print("jinja2 not found. Try `pip install jinja2`")
         sys.exit(1)
